--[[
Author: Starinnia
RealID Toons - Add character information to the BNet alerts your RealID friends generate
$Date: 2010-09-05 21:18:04 +0000 (Sun, 05 Sep 2010) $
$Revision: 64 $
Project Version: r66
contact: codemaster2010 AT gmail DOT com

Copyright (c) 2010 Michael J. Murray aka Lyte of Lothar(US)
All rights reserved unless otherwise explicitly stated.
]]

local format = string.format

--save for the future
local myrealm = GetRealmName()

--create a reverse lookup from localized classes to english names
--we need the english names to get the class colors
local reverseClassLookup = {}
for k, v in pairs(LOCALIZED_CLASS_NAMES_FEMALE) do
	reverseClassLookup[v] = k
end
for k, v in pairs(LOCALIZED_CLASS_NAMES_MALE) do
	reverseClassLookup[v] = k
end

--pass in the toonID of the friend
--from this we can obtain the character name, realm, class, and faction
local function constructToonName(toonID, presenceID, isChat)
	local _, toon, _, realm, faction, _, class = BNGetToonInfo(toonID)
	local note = select(12, BNGetFriendInfoByID(presenceID))
	local canCoop = CanCooperateWithToon(toonID)
	
	if RID_TOONS_COLOR then
		--pass the localized class into the lookup table
		local eClass = reverseClassLookup[class]
		local r, g, b
		if _G.CUSTOM_CLASS_COLORS and _G.CUSTOM_CLASS_COLORS[eClass] then
			r, g, b = CUSTOM_CLASS_COLORS[eClass].r*255, CUSTOM_CLASS_COLORS[eClass].g*255, CUSTOM_CLASS_COLORS[eClass].b*255
		elseif RAID_CLASS_COLORS[eClass] then
			r, g, b = RAID_CLASS_COLORS[eClass].r*255, RAID_CLASS_COLORS[eClass].g*255, RAID_CLASS_COLORS[eClass].b*255
		else
			--sometimes BNGetToonInfo does not return class info
			--when this occurs default to a gray color
			r, g, b = 0.63*255, 0.63*255, 0.63*255
		end
		
		if isChat and canCoop then
			toon = format("|Hplayer:%s|h|cff%02x%02x%02x%s|r|h", toon, r, g, b, toon)
		else
			toon = format("|cff%02x%02x%02x%s|r", r, g, b, toon)
		end
	end
	
	--make sure realm has a value, sometimes BNGetToonInfo returns an empty string
	if RID_TOONS_REALM and realm ~= myrealm and realm ~= "" then
		toon = toon.." - "..realm
	end
	
	if RID_TOONS_FACTION then
		if faction and (not canCoop) then
			if faction == 0 then
				toon = format("|TInterface\\PVPFrame\\PVP-Currency-Horde:14|t%s", toon)
			elseif faction == 1 then
				toon = format("|TInterface\\PVPFrame\\PVP-Currency-Alliance:16|t%s", toon)
			end
		end
	end
	
	if RID_TOONS_SHOWNOTE then
		if note and note ~= "" then
			toon = format("%s [%s]", toon, note)
		end
	end
	
	return toon
end

local function filter(self, event, msg, author, ...)
	--the only toast we care about is FRIEND_ONLINE
	if msg ~= "FRIEND_ONLINE" then
		return false, msg, author, ...
	end
	
	local presenceID, givenName, surname, toonName, toonID, client, fullName
	local type = strsub(event, 10)
	local info = ChatTypeInfo[type]
	local arg11 = select(9, ...)
	
	--loop through the BNet friends to determine the toon name
	for i = 1, BNGetNumFriends() do
		presenceID, givenName, surname, toonName, toonID, client = BNGetFriendInfo(i)
		
		--if we can't obtain the data, bail out and let the default alert through
		if not givenName or not surname then
			break
		end
		
		fullName = format(BATTLENET_NAME_FORMAT, givenName, surname)
		--only attempt to get character names from WoW clients
		if author == fullName and client == BNET_CLIENT_WOW then
			--construct the playerlink and message prefix seperately for easier debugging
			local playerlink = format("|HBNplayer:%s:%s:%s:%s:%s|h[%s]|h", author, presenceID, arg11, Chat_GetChatCategory(type), 0, author)
			local prefix = format(BN_INLINE_TOAST_FRIEND_ONLINE, playerlink)
			--add a message to the passed in chat frame that looks like the regular toast
			self:AddMessage(format("%s (%s)", prefix, constructToonName(toonID, presenceID, true)), info.r, info.g, info.b, info.id)
			
			--block the actual toast, it is replaced with our own
			return true, msg, author, ...
			
		--if the client is SC2 add the SC2 icon after the real name
		elseif author == fullName and client == BNET_CLIENT_SC2 then
			--construct the playerlink and message prefix seperately for easier debugging
			local playerlink = format("|HBNplayer:%s:%s:%s:%s:%s|h[%s]|h", author, presenceID, arg11, Chat_GetChatCategory(type), 0, author)
			local prefix = format(BN_INLINE_TOAST_FRIEND_ONLINE, playerlink)
			
			self:AddMessage(format("%s (|TInterface\\FriendsFrame\\Battlenet-Sc2icon:17|t %s)", prefix, toonName), info.r, info.g, info.b, info.id)
			--again, block the actual toast, we replace it with our customized message
			return true, msg, author, ...
		end
	end
	
	--if we reach here there was a data issue or an unknown client, so we allow the default alert through
	return false, msg, author, ...
end
ChatFrame_AddMessageEventFilter("CHAT_MSG_BN_INLINE_TOAST_ALERT", filter)

hooksecurefunc("BNToastFrame_Show", function()
	BNToastFrame:SetWidth(250) -- Need to reset to original value for popups we are not modifying
	BNToastFrameGlowFrame.glow:SetWidth(252)
	if BNToastFrame.toastType ~= 1 then return end
	
	local _, _, _, toon, toonID, client = BNGetFriendInfoByID(BNToastFrame.toastData)
	
	if client == BNET_CLIENT_WOW then
		local originalText = BNToastFrameTopLine:GetText()
		local toonName = constructToonName(toonID, BNToastFrame.toastData, false)
		BNToastFrameTopLine:SetWidth(0)
		BNToastFrameTopLine:SetFormattedText("%s (%s)", originalText, toonName)
		--size the popup to match the text
		if (BNToastFrameTopLine:GetStringWidth() + 15) > (BN_TOAST_MAX_LINE_WIDTH - 10) then
			BNToastFrameTopLine:SetWidth(BNToastFrameTopLine:GetStringWidth() + 30)
			BNToastFrame:SetWidth(45+BNToastFrameTopLine:GetWidth())
			--make the animation glow effect fit the resized Toast popup
			BNToastFrameGlowFrame.glow:SetWidth(BNToastFrame:GetWidth()+2)
		end
	elseif client == BNET_CLIENT_SC2 then
		local originalText = BNToastFrameTopLine:GetText()
		BNToastFrameTopLine:SetWidth(0)
		BNToastFrameTopLine:SetFormattedText("%s - |TInterface\\FriendsFrame\\Battlenet-Sc2icon:17|t %s", originalText, toon)
		--size the popup to match the text
		if (BNToastFrameTopLine:GetStringWidth() + 15) > (BN_TOAST_MAX_LINE_WIDTH - 10) then
			BNToastFrameTopLine:SetWidth(BNToastFrameTopLine:GetStringWidth() + 30)
			BNToastFrame:SetWidth(45+BNToastFrameTopLine:GetWidth())
			--make the animation glow effect fit the resized Toast popup
			BNToastFrameGlowFrame.glow:SetWidth(BNToastFrame:GetWidth()+2)
		end
	end
end)
